@@ -7,6 +7,8 @@
 """
 # pylint: disable=no-self-use
 
+from re import match
+
 from SoftLayer import exceptions
 
 CATEGORY_MASK = '''id,
@@ -16,7 +18,7 @@ CATEGORY_MASK = '''id,
 
 ITEM_MASK = '''id, keyName, description, itemCategory, categories'''
 
-PACKAGE_MASK = '''id, name, keyName, isActive'''
+PACKAGE_MASK = '''id, name, keyName, isActive, type'''
 
 PRESET_MASK = '''id, name, keyName, description'''
 
@@ -297,7 +299,18 @@ class OrderingManager(object):
     def get_preset_by_key(self, package_keyname, preset_keyname, mask=None):
         """Gets a single preset with the given key."""
         preset_operation = '_= %s' % preset_keyname
-        _filter = {'activePresets': {'keyName': {'operation': preset_operation}}}
+        _filter = {
+            'activePresets': {
+                'keyName': {
+                    'operation': preset_operation
+                }
+            },
+            'accountRestrictedActivePresets': {
+                'keyName': {
+                    'operation': preset_operation
+                }
+            }
+        }
 
         presets = self.list_presets(package_keyname, mask=mask, filter=_filter)
 
@@ -321,10 +334,11 @@ class OrderingManager(object):
                   keynames in the given package
 
         """
-        mask = 'id, keyName, prices'
+        mask = 'id, itemCategory, keyName, prices[categories]'
         items = self.list_items(package_keyname, mask=mask)
 
         prices = []
+        gpu_number = -1
         for item_keyname in item_keynames:
             try:
                 # Need to find the item in the package that has a matching
@@ -340,8 +354,17 @@ class OrderingManager(object):
             # because that is the most generic price. verifyOrder/placeOrder
             # can take that ID and create the proper price for us in the location
             # in which the order is made
-            price_id = [p['id'] for p in matching_item['prices']
-                        if p['locationGroupId'] == ''][0]
+            if matching_item['itemCategory']['categoryCode'] != "gpu0":
+                price_id = [p['id'] for p in matching_item['prices']
+                            if not p['locationGroupId']][0]
+            else:
+                # GPU items has two generic prices and they are added to the list
+                # according to the number of gpu items added in the order.
+                gpu_number += 1
+                price_id = [p['id'] for p in matching_item['prices']
+                            if not p['locationGroupId']
+                            and p['categories'][0]['categoryCode'] == "gpu" + str(gpu_number)][0]
+
             prices.append(price_id)
 
         return prices
@@ -432,6 +455,7 @@ class OrderingManager(object):
         :param int quantity: The number of resources to order
 
         """
+        container = {}
         order = {}
         extras = extras or {}
 
@@ -443,7 +467,7 @@ class OrderingManager(object):
         #                                    'domain': 'softlayer.com'}]}
         order.update(extras)
         order['packageId'] = package['id']
-        order['location'] = location
+        order['location'] = self.get_location_id(location)
         order['quantity'] = quantity
         order['useHourlyPricing'] = hourly
 
@@ -457,7 +481,10 @@ class OrderingManager(object):
 
         price_ids = self.get_price_id_list(package_keyname, item_keynames)
         order['prices'] = [{'id': price_id} for price_id in price_ids]
-        return order
+
+        container['orderContainers'] = [order]
+
+        return container
 
     def package_locations(self, package_keyname):
         """List datacenter locations for a package keyname
@@ -471,3 +498,23 @@ class OrderingManager(object):
 
         regions = self.package_svc.getRegions(id=package['id'], mask=mask)
         return regions
+
+    def get_location_id(self, location):
+        """Finds the location ID of a given datacenter
+
+        This is mostly used so either a dc name, or regions keyname can be used when ordering
+        :param str location: Region Keyname (DALLAS13) or datacenter name (dal13)
+        :returns: integer id of the datacenter
+        """
+
+        if isinstance(location, int):
+            return location
+        mask = "mask[id,name,regions[keyname]]"
+        if match(r'[a-zA-Z]{3}[0-9]{2}', location) is not None:
+            search = {'name': {'operation': location}}
+        else:
+            search = {'regions': {'keyname': {'operation': location}}}
+        datacenter = self.client.call('SoftLayer_Location', 'getDatacenters', mask=mask, filter=search)
+        if len(datacenter) != 1:
+            raise exceptions.SoftLayerError("Unable to find location: %s" % location)
+        return datacenter[0]['id']
